@@ -1,16 +1,19 @@
-use crate::parser::parol_grammar_trait::ParolGrammarTrait;
+use super::parol_grammar_trait::ParolGrammarTrait;
+use super::ParolParserError;
 use id_tree::Tree;
 use log::trace;
-use miette::{miette, Result};
+use miette::{miette, IntoDiagnostic, Result};
+use parol_runtime::errors::FileSource;
 use parol_runtime::parser::{ParseTreeStackEntry, ParseTreeType};
 use std::fmt::{Debug, Display, Error, Formatter};
+use std::path::PathBuf;
 
 // To rebuild the parser sources from scratch use the command build_parsers.ps1
 
 // Test run:
 // parol -f .\src\parser\parol-grammar.par -v
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub enum Factor {
     Group(Alternations),
     Repeat(Alternations),
@@ -45,7 +48,11 @@ impl Display for Factor {
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+///
+/// An Alternation is a sequence of factors.
+/// Valid operation on Alternation is "|".
+///
+#[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Alternation(pub Vec<Factor>);
 
 impl Display for Alternation {
@@ -67,16 +74,16 @@ impl Alternation {
         Self(Vec::new())
     }
 
+    pub(crate) fn insert(&mut self, fac: Factor) {
+        self.0.insert(0, fac)
+    }
+
     pub(crate) fn push(&mut self, fac: Factor) {
         self.0.push(fac)
     }
-
-    fn reverse(&mut self) {
-        self.0.reverse()
-    }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Alternations(pub Vec<Alternation>);
 
 impl Alternations {
@@ -84,12 +91,8 @@ impl Alternations {
         Self(Vec::new())
     }
 
-    fn push(&mut self, alt: Alternation) {
-        self.0.push(alt)
-    }
-
-    fn reverse(&mut self) {
-        self.0.reverse()
+    pub(crate) fn insert(&mut self, alt: Alternation) {
+        self.0.insert(0, alt)
     }
 }
 
@@ -107,7 +110,7 @@ impl Display for Alternations {
     }
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Production {
     pub lhs: String,
     pub rhs: Alternations,
@@ -125,7 +128,7 @@ impl Display for Production {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ScannerConfig {
     pub name: String,
     pub line_comments: Vec<String>,
@@ -134,7 +137,7 @@ pub struct ScannerConfig {
     pub auto_ws_off: bool,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub enum ParolGrammarItem {
     Prod(Production),
     Alts(Alternations),
@@ -187,7 +190,7 @@ impl Default for ScannerConfig {
 ///
 /// Data structure used to build up a parol::GrammarConfig during parsing.
 ///
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ParolGrammar {
     pub item_stack: Vec<ParolGrammarItem>,
     pub title: Option<String>,
@@ -195,6 +198,7 @@ pub struct ParolGrammar {
     pub start_symbol: String,
     pub scanner_configurations: Vec<ScannerConfig>,
     current_scanner: ScannerConfig,
+    file_name: PathBuf,
 }
 
 impl Default for ParolGrammar {
@@ -206,6 +210,7 @@ impl Default for ParolGrammar {
             start_symbol: String::default(),
             scanner_configurations: vec![ScannerConfig::default()],
             current_scanner: ScannerConfig::default(),
+            file_name: PathBuf::default(),
         }
     }
 }
@@ -233,7 +238,12 @@ impl ParolGrammar {
         )
     }
 
-    fn handle_scanner_state(&mut self, context: &str) -> Result<()> {
+    fn handle_scanner_state(
+        &mut self,
+        context: &str,
+        identifier_0: &ParseTreeStackEntry,
+        parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
         let l = self.pop(context);
         let s = self.pop(context);
 
@@ -252,6 +262,19 @@ impl ParolGrammar {
                     self.push(ParolGrammarItem::StateList(l), context);
                     trace!("{}", self.trace_item_stack(context));
                     Ok(())
+                } else if let ParseTreeStackEntry::Id(node_id) = identifier_0 {
+                    // We need to navigate to the one and only child of the Identifier
+                    // non-terminal to access the actual token.
+                    let child = parse_tree
+                        .get(node_id)
+                        .and_then(|node_ref| parse_tree.get(&node_ref.children()[0]))
+                        .into_diagnostic()?;
+                    Err(miette!(ParolParserError::UnknownScanner {
+                        context: context.to_owned(),
+                        name: s.clone(),
+                        input: FileSource::try_new(self.file_name.clone())?.into(),
+                        token: child.data().token()?.into()
+                    }))
                 } else {
                     Err(miette!("{}: Unknown scanner name '{}'", context, s))
                 }
@@ -311,6 +334,13 @@ impl Display for ParolGrammar {
 }
 
 impl ParolGrammarTrait for ParolGrammar {
+    ///
+    /// Information provided by parser
+    ///
+    fn init(&mut self, file_name: &std::path::Path) {
+        self.file_name = file_name.into();
+    }
+
     /// Semantic action for production 2:
     ///
     /// StartDeclaration: "%start" Identifier;
@@ -482,9 +512,8 @@ impl ParolGrammarTrait for ParolGrammar {
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "production_17";
-        if let Some(ParolGrammarItem::Alts(mut rhs)) = self.pop(context) {
+        if let Some(ParolGrammarItem::Alts(rhs)) = self.pop(context) {
             if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(lhs))) = self.pop(context) {
-                rhs.reverse();
                 self.push(ParolGrammarItem::Prod(Production::new(lhs, rhs)), context);
                 Ok(())
             } else {
@@ -510,9 +539,8 @@ impl ParolGrammarTrait for ParolGrammar {
     ) -> Result<()> {
         let context = "alternations_18";
         if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
-            if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
-                alt.reverse();
-                alts.push(alt);
+            if let Some(ParolGrammarItem::Alt(alt)) = self.pop(context) {
+                alts.insert(alt);
                 self.push(ParolGrammarItem::Alts(alts), context);
                 Ok(())
             } else {
@@ -536,9 +564,8 @@ impl ParolGrammarTrait for ParolGrammar {
     ) -> Result<()> {
         let context = "alternations_list_19";
         if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
-            if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
-                alt.reverse();
-                alts.push(alt);
+            if let Some(ParolGrammarItem::Alt(alt)) = self.pop(context) {
+                alts.insert(alt);
                 self.push(ParolGrammarItem::Alts(alts), context);
                 Ok(())
             } else {
@@ -573,7 +600,7 @@ impl ParolGrammarTrait for ParolGrammar {
         //trace!("{}", self.trace_item_stack(context));
         if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
             if let Some(ParolGrammarItem::Fac(fac)) = self.pop(context) {
-                alt.push(fac);
+                alt.insert(fac);
                 self.push(ParolGrammarItem::Alt(alt), context);
                 Ok(())
             } else {
@@ -622,29 +649,28 @@ impl ParolGrammarTrait for ParolGrammar {
 
     /// Semantic action for production 34:
     ///
-    /// Group: "\(" Factor Alternations "\)";
+    /// Group: "\(" Alternations "\)";
     ///
     fn group_34(
         &mut self,
-        _l_paren_0: &ParseTreeStackEntry,
-        _factor_1: &ParseTreeStackEntry,
-        _alternations_2: &ParseTreeStackEntry,
-        _r_paren_3: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
+        l_paren_0: &ParseTreeStackEntry,
+        _alternations_1: &ParseTreeStackEntry,
+        r_paren_2: &ParseTreeStackEntry,
+        parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "group_34";
         trace!("{}", self.trace_item_stack(context));
-        if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
-            if let Some(ParolGrammarItem::Fac(fac)) = self.pop(context) {
-                if alts.0.is_empty() {
-                    alts.push(Alternation(vec![fac]));
-                } else {
-                    alts.0[0].0.insert(0, fac);
-                }
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            if alts.0.is_empty() || (alts.0.len() == 1 && alts.0[0].0.is_empty()) {
+                Err(miette!(ParolParserError::EmptyGroup {
+                    context: context.to_owned(),
+                    input: FileSource::try_new(self.file_name.clone())?.into(),
+                    start: l_paren_0.token(parse_tree)?.into(),
+                    end: r_paren_2.token(parse_tree)?.into(),
+                }))
+            } else {
                 self.push(ParolGrammarItem::Fac(Factor::Group(alts)), context);
                 Ok(())
-            } else {
-                Err(miette!("{}: Expected 'Factor' on TOS.", context))
             }
         } else {
             Err(miette!("{}: Expected 'Alts' on TOS.", context))
@@ -653,29 +679,28 @@ impl ParolGrammarTrait for ParolGrammar {
 
     /// Semantic action for production 35:
     ///
-    /// Optional: "\[" Factor Alternations "\]";
+    /// Optional: "\[" Alternations "\]";
     ///
     fn optional_35(
         &mut self,
-        _l_bracket_0: &ParseTreeStackEntry,
-        _factor_1: &ParseTreeStackEntry,
-        _alternations_2: &ParseTreeStackEntry,
-        _r_bracket_3: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
+        l_bracket_0: &ParseTreeStackEntry,
+        _alternations_1: &ParseTreeStackEntry,
+        r_bracket_2: &ParseTreeStackEntry,
+        parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "optional_35";
         trace!("{}", self.trace_item_stack(context));
-        if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
-            if let Some(ParolGrammarItem::Fac(fac)) = self.pop(context) {
-                if alts.0.is_empty() {
-                    alts.push(Alternation(vec![fac]));
-                } else {
-                    alts.0[0].0.insert(0, fac);
-                }
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            if alts.0.is_empty() || (alts.0.len() == 1 && alts.0[0].0.is_empty()) {
+                Err(miette!(ParolParserError::EmptyOptional {
+                    context: context.to_owned(),
+                    input: FileSource::try_new(self.file_name.clone())?.into(),
+                    start: l_bracket_0.token(parse_tree)?.into(),
+                    end: r_bracket_2.token(parse_tree)?.into(),
+                }))
+            } else {
                 self.push(ParolGrammarItem::Fac(Factor::Optional(alts)), context);
                 Ok(())
-            } else {
-                Err(miette!("{}: Expected 'Factor' on TOS.", context))
             }
         } else {
             Err(miette!("{}: Expected 'Alts' on TOS.", context))
@@ -684,29 +709,28 @@ impl ParolGrammarTrait for ParolGrammar {
 
     /// Semantic action for production 36:
     ///
-    /// Repeat: "\{" Factor Alternations "\}";
+    /// Repeat: "\{" Alternations "\}";
     ///
     fn repeat_36(
         &mut self,
-        _l_brace_0: &ParseTreeStackEntry,
-        _factor_1: &ParseTreeStackEntry,
-        _alternations_2: &ParseTreeStackEntry,
-        _r_brace_3: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
+        l_brace_0: &ParseTreeStackEntry,
+        _alternations_1: &ParseTreeStackEntry,
+        r_brace_2: &ParseTreeStackEntry,
+        parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "repeat_36";
         trace!("{}", self.trace_item_stack(context));
-        if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
-            if let Some(ParolGrammarItem::Fac(fac)) = self.pop(context) {
-                if alts.0.is_empty() {
-                    alts.push(Alternation(vec![fac]));
-                } else {
-                    alts.0[0].0.insert(0, fac);
-                }
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            if alts.0.is_empty() || (alts.0.len() == 1 && alts.0[0].0.is_empty()) {
+                Err(miette!(ParolParserError::EmptyRepetition {
+                    context: context.to_owned(),
+                    input: FileSource::try_new(self.file_name.clone())?.into(),
+                    start: l_brace_0.token(parse_tree)?.into(),
+                    end: r_brace_2.token(parse_tree)?.into(),
+                }))
+            } else {
                 self.push(ParolGrammarItem::Fac(Factor::Repeat(alts)), context);
                 Ok(())
-            } else {
-                Err(miette!("{}: Expected 'Factor' on TOS.", context))
             }
         } else {
             Err(miette!("{}: Expected 'Alts' on TOS.", context))
@@ -801,13 +825,13 @@ impl ParolGrammarTrait for ParolGrammar {
     ///
     fn state_list_42(
         &mut self,
-        _identifier_0: &ParseTreeStackEntry,
+        identifier_0: &ParseTreeStackEntry,
         _state_list_list_1: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
+        parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "state_list_42";
         trace!("{}", self.trace_item_stack(context));
-        self.handle_scanner_state(context)
+        self.handle_scanner_state(context, identifier_0, parse_tree)
     }
 
     /// Semantic action for production 43:
@@ -817,13 +841,13 @@ impl ParolGrammarTrait for ParolGrammar {
     fn state_list_rest_43(
         &mut self,
         _comma_0: &ParseTreeStackEntry,
-        _identifier_1: &ParseTreeStackEntry,
+        identifier_1: &ParseTreeStackEntry,
         _state_list_rest_2: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
+        parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "state_list_rest_43";
         trace!("{}", self.trace_item_stack(context));
-        self.handle_scanner_state(context)
+        self.handle_scanner_state(context, identifier_1, parse_tree)
     }
 
     /// Semantic action for production 44:
